require 'parametric'

class Stringer < Parametric

  def default_parameters
    defaults = {
      'rise'            => 7.75,
      'run'             => 10.0,
      'steps'           => 7,
      'tread_thickness' => 1.0
    }
  end

  def translate_key(key)
    prompt = case(key)
             when 'rise'
               'Rise'
             when 'run'
               'Run'
             when 'steps'
               'Number of steps'
             when 'tread_thickness'
               'Tread Thickness '
             end
  end

  def self.create
    stringer = Stringer.new
    defn = stringer.entity
    Sketchup.active_model.place_component(defn, true)
  end

  def create_entity(model)
    @entity = model.definitions.add("Stringer")
  end

  def create_entities(data, container)
    draw_stringer(data, container)
    # draw_treads(data, container)
  end

  def draw_stringer(data, container)

    rise            = data['rise']
    run             = data['run']
    tread_thickness = data['tread_thickness']
    numsteps        = data['steps']
    string          = 11
    rise_thickness  = 0.5

    # offset vectors for rise and run
    risevect = Geom::Vector3d.new([0,0,0], [0,0,rise])
    runvect  = Geom::Vector3d.new([0,0,0], [0,run,0])

    lstrun = Geom::Vector3d.new([0,0,0], [0,(run-rise_thickness),0])

    # arbitrarily start at the global SketchUp origin
    basepoint = ORIGIN
    # draw the upper profile of the stringer
    # first step adjusted for tread thickness
    corner = basepoint.offset(Geom::Vector3d.new([0,0,0], [0,0,rise-tread_thickness]))
    container.add_line(basepoint, corner)
    # remember first corner for later
    firstcorner = corner
    basepoint = corner.offset(runvect)
    container.add_line(corner, basepoint)

    # now do the rest of the steps
    2.upto(numsteps-2) do
      corner = basepoint.offset(risevect)
      container.add_line(basepoint, corner)
      basepoint = corner.offset(runvect)
      container.add_line(corner, basepoint)
    end
    # <<<<<<<<<<<<last run
    corner = basepoint.offset(risevect)
    container.add_line(basepoint, corner)
    basepoint = corner.offset(lstrun)
    container.add_line(corner, basepoint)

    # create the baseline and top plumb line
    baseline = [ORIGIN, [0,1,0]]
    plumbline = [basepoint, [0,0,-1]]

    # create the offset vector to the other line
    cornervect = Geom::Vector3d.new(firstcorner, corner)
    normalvect = cornervect*[0,0,1]
    offsetvect = cornervect*normalvect
    offsetvect.length=string

    # create the two corners for the remaining Edges
    # get a point on the bottom line by offsetting from any top corner
    refpoint = corner.offset(offsetvect)
    oppline = [refpoint, cornervect]
    bottomcorner = Geom::intersect_line_line(oppline, baseline)
    topcorner = Geom::intersect_line_line(oppline, plumbline)

    # add the Edges
    container.add_line(ORIGIN, bottomcorner)
    container.add_line(bottomcorner, topcorner)
    finaledge = container.add_line(topcorner, basepoint)

    # fill in the Face
    pre_faces = container.grep(Sketchup::Face)
    finaledge.find_faces()
    post_faces = container.grep(Sketchup::Face)
    faces = post_faces - pre_faces
    faces.first.pushpull(tread_thickness)
  end

end

UI.menu('Draw').add_item('Stringer') {
  Stringer.create
}
